@@ -24,6 +24,7 @@ type Snapshot struct {
 	command string
 	args    []string
 
+	noShell   bool
 	shell     string
 	shellOpts string
 
@@ -47,14 +48,16 @@ type Snapshot struct {
 	finish chan<- struct{}
 }
 
-//nolint:lll
 // NewSnapshot returns Snapshot object.
-func NewSnapshot(id int64, command string, args []string, shell string, shellOpts string, before *Snapshot, finish chan<- struct{}) *Snapshot {
+//
+//nolint:lll
+func NewSnapshot(id int64, command string, args []string, noShell bool, shell string, shellOpts string, before *Snapshot, finish chan<- struct{}) *Snapshot {
 	return &Snapshot{
 		id:      id,
 		command: command,
 		args:    args,
 
+		noShell:   noShell,
 		shell:     shell,
 		shellOpts: shellOpts,
 
@@ -96,22 +99,25 @@ func (s *Snapshot) compareFromBefore() error {
 	return nil
 }
 
+//nolint:gosec
 func (s *Snapshot) prepareCommand(commands []string) *exec.Cmd {
-	var command *exec.Cmd
-
 	if runtime.GOOS == "windows" {
 		cmdStr := strings.Join(commands, " ")
 		compSec := os.Getenv("COMSPEC")
-		command = exec.Command(compSec, "/c", cmdStr)
-	} else {
-		var args []string
-		args = append(args, strings.Fields(s.shellOpts)...)
-		args = append(args, "-c")
-		args = append(args, strings.Join(commands, " "))
-		command = exec.Command(s.shell, args...) //nolint:gosec
+
+		return exec.Command(compSec, "/c", cmdStr)
 	}
 
-	return command
+	if s.noShell {
+		return exec.Command(commands[0], commands[1:]...)
+	}
+
+	var args []string
+	args = append(args, strings.Fields(s.shellOpts)...)
+	args = append(args, "-c")
+	args = append(args, strings.Join(commands, " "))
+
+	return exec.Command(s.shell, args...)
 }
 
 func isWhiteString(str string) bool {
@@ -168,7 +174,7 @@ func DiffPrettyText(diffs []diffmatchpatch.Diff) string {
 				if unicode.IsSpace(c) {
 					_, _ = buff.WriteRune(c)
 				} else {
-					_, _ = buff.WriteString(color.New(color.BgGreen).Sprintf(string(c)))
+					_, _ = buff.WriteString(color.New(color.FgBlack).Sprintf(color.New(color.BgGreen).Sprintf(string(c))))
 				}
 			}
 		case diffmatchpatch.DiffEqual:
